from steel.facades.texts.random_text_generation \
    import                                      \
    generate_lowercase_label_by_size,           \
    generate_uppercase_label_by_size,           \
    generate_label_by_size




from steel.facades.texts.characters     \
    import                              \
    random_lowercase_letter,            \
    random_uppercase_letter,            \
    random_letter


def generate_lowercase_label_by_size(
        length_of_label: int
) -> str:
    r_value: str = ''

    for position                        \
            in range(length_of_label):

        r_value = r_value + random_lowercase_letter()

    return r_value


def generate_uppercase_label_by_size(
        length_of_label: int
) -> str:
    r_value: str = ''

    for position                        \
            in range(length_of_label):

        r_value = r_value + random_uppercase_letter()

    return r_value


def generate_label_by_size(
        length_of_label: int
) -> str:
    r_value: str = ''

    for position                        \
            in range(length_of_label):
        r_value = r_value + random_letter()

    return r_value

















from random \
    import SystemRandom

singleton_system_random: SystemRandom | None = None


def get_system_random() -> SystemRandom:
    global singleton_system_random

    if is_system_random_none():
        set_system_random(
            SystemRandom()
        )

    return singleton_system_random


def set_system_random(
        value: SystemRandom
) -> None:
    global singleton_system_random
    singleton_system_random = value


def is_system_random_none() -> bool:
    global singleton_system_random
    return singleton_system_random is None



from steel.globals                          \
    import get_system_random

from steel.facades.texts.characters.ranges  \
    import                                  \
    random_lowercase_letter,                \
    random_uppercase_letter,                \
    random_letter



from random \
    import SystemRandom

from steel.globals \
    import \
    get_system_random, \
    set_system_random


def test_get() -> None:
    assert not (get_system_random() is None)


def test_set() -> None:
    set_system_random(
        SystemRandom()
    )

    assert not (get_system_random() is None)



from steel.facades.texts.characters \
    import get_system_random

ascii_lowercase_begin:  int = ord('a')
ascii_lowercase_end:    int = ord('z')

ascii_uppercase_begin:  int = ord('A')
ascii_uppercase_end:    int = ord('Z')


def random_letter() -> chr:
    random = get_system_random()

    choose_uppercase: bool = bool(
        random.getrandbits(1)
    )

    if choose_uppercase:
        return random_uppercase_letter()
    else:
        return random_lowercase_letter()


def random_uppercase_letter() -> chr:
    global                          \
        ascii_uppercase_begin,      \
        ascii_uppercase_end

    return character_in_range(
        ascii_uppercase_begin,
        ascii_uppercase_end
    )


def random_lowercase_letter() -> chr:
    global                          \
        ascii_lowercase_begin,      \
        ascii_lowercase_end

    return character_in_range(
        ascii_lowercase_begin,
        ascii_lowercase_end
    )


def character_in_range(
        begin: int,
        end: int
) -> chr:
    random = get_system_random()

    random_number: int = random.randint(
        begin,
        end
    )

    return chr(
        random_number
    )
